/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.Web;
using System.Data;
using System.Web.UI;
using System.Collections;
using System.ComponentModel;
using System.Security.Permissions;
using TopCoder.Util.ObjectFactory;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines a web control which can render the bread crumb trail of the web page.
    /// </summary>
    /// <remarks>
    /// This web control uses an <c>INodeDiscovery</c> to discover the current bread crumb node of the current web page.
    /// It then uses an <c>IPathDiscovery</c> to find out the bread crumb trail for the current bread crumb node.
    /// Finally, it uses an <c>IPathWriter</c> to render the retrieved bread crumb trail to the web page. It also
    /// provides possibility to override the bread crumb trail by setting the data source as a data set, a data table or
    /// a list of <c>BreadCrumbNode</c> instances. The <c>INodeDiscovery</c> instance, <c>IPathDiscovery</c> instance
    /// and <c>IPathWriter</c> instance are created by the Object Factory component, which is configurable. All
    /// properties can be configured using the configuration manager. Under the namespace
    /// 'TopCoder.Web.UI.WebControl.BreadCrumb.BreadCrumbTrail', property 'pathSeparator', 'pathSeparatorStyle',
    /// 'nodeStyle', 'rootNodeStyle', 'currentNodeStyle', 'mouseOverStyle', 'pathDirection', 'errorDisposition',
    /// 'nodeDiscoveryKey', 'pathDiscoveryKey', and 'pathWriterKey' define their respective properties. This class
    /// is not thread-safe. However, during rendering stage, it can be called by multiple threads.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [AspNetHostingPermission(SecurityAction.Demand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [DefaultProperty("PathSeparator")]
    [ToolboxData("<{0}:BreadCrumbTrailControl runat=\"server\"> </{0}:BreadCrumbTrailControl>")]
    [Designer(typeof(BreadCrumbTrailControlDesigner))]
    public class BreadCrumbTrailControl : System.Web.UI.WebControls.WebControl
    {
        /// <summary>
        /// Represents the column name in the data table where the title of the page is stored.
        /// </summary>
        private const string TITLE_COLUMN = "Title";

        /// <summary>
        /// Represents the column name in the data table where the URL of the page is stored.
        /// </summary>
        private const string URL_COLUMN = "Url";

        /// <summary>
        /// Represents the property name where the path separator is stored in configuration.
        /// </summary>
        private const string PATHSEPARATOR_PROPERTY = "pathSeparator";

        /// <summary>
        /// Represents the property name where the path separator CSS style class is stored in configuration.
        /// </summary>
        private const string PATHSEPARATORSTYLE_PROPERTY = "pathSeparatorStyle";

        /// <summary>
        /// Represents the property name where the normal node CSS style class is stored in configuration.
        /// </summary>
        private const string NODESTYLE_PROPERTY = "nodeStyle";

        /// <summary>
        /// Represents the property name where the root node CSS style class is stored in configuration.
        /// </summary>
        private const string ROOTNODESTYLE_PROPERTY = "rootNodeStyle";

        /// <summary>
        /// Represents the property name where the current node CSS style class is stored in configuration.
        /// </summary>
        private const string CURRENTNODESTYLE_PROPERTY = "currentNodeStyle";

        /// <summary>
        /// Represents the property name where the node CSS style class when mouse hovers over is stored in
        /// configuration.
        /// </summary>
        private const string MOUSEOVERSTYLE_PROPERTY = "mouseOverStyle";

        /// <summary>
        /// Represents the property name where the path direction flag is stored in configuration.
        /// </summary>
        private const string PATHDIRECTION_PROPERTY = "pathDirection";

        /// <summary>
        /// Represents the property name where the error disposition flag is stored in configuration.
        /// </summary>
        private const string ERRORDISPOSITION_PROPERTY = "errorDisposition";

        /// <summary>
        /// Represents the property name where the key used to create <c>INodeDiscovery</c> instance is stored in
        /// configuration.
        /// </summary>
        private const string NODEDISCOVERKEY_PROPERTY = "nodeDiscoveryKey";

        /// <summary>
        /// Represents the property name where the key used to create <c>IPathDiscovery</c> instance is stored in
        /// configuration.
        /// </summary>
        private const string PATHDISCOVERKEY_PROPERTY = "pathDiscoveryKey";

        /// <summary>
        /// Represents the property name where the key used to create <c>IPathWriter</c> instance is stored in
        /// configuration.
        /// </summary>
        private const string PATHWRITERKEY_PROPERTY = "pathWriterKey";

        /// <summary>
        /// Gets or sets the path separator.
        /// </summary>
        /// <remarks>
        /// The path separator is rendered between two bread crumb nodes in the trail. If it is <c>null</c>, there is
        /// no path separator.
        /// </remarks>
        /// <value>The path separator.</value>
        [Bindable(true), Category("Appearance"), DefaultValue(":"), Description("The path separator to use.")]
        public virtual string PathSeparator
        {
            get
            {
                return pathSeparator;
            }
            set
            {
                pathSeparator = (value == null) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the CSS style class for the path separator.
        /// </summary>
        /// <remarks>
        /// When the CSS style class is <c>null</c> or empty string, there is no CSS style class.
        /// </remarks>
        /// <value>The CSS style class for the path separator.</value>
        [Bindable(true), Category("Appearance"), DefaultValue("")]
        [Description("The CSS Style Class to use for the path separator.")]
        public virtual string PathSeparatorStyle
        {
            get
            {
                return pathSeparatorStyle;
            }
            set
            {
                pathSeparatorStyle = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the CSS style class for the bread crumb node in the trail.
        /// </summary>
        /// <remarks>
        /// When the CSS style class is <c>null</c> or empty string, there is no CSS style class.
        /// </remarks>
        /// <value>The CSS style class for the bread crumb node.</value>
        [Bindable(true), Category("Appearance"), DefaultValue("")]
        [Description("The CSS Style Class to use for each node in the path.")]
        public virtual string NodeStyle
        {
            get
            {
                return nodeStyle;
            }
            set
            {
                nodeStyle = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the CSS style class for the current bread crumb node in the trail.
        /// </summary>
        /// <remarks>
        /// When the CSS style class is <c>null</c> or empty string, the default bread crumb node CSS style class is
        /// used if available.
        /// </remarks>
        /// <value>The CSS style class for the current bread crumb node.</value>
        [Bindable(true), Category("Appearance"), DefaultValue("")]
        [Description("The CSS Style Class to use for the current node in the path.")]
        public virtual string CurrentNodeStyle
        {
            get
            {
                return currentNodeStyle;
            }
            set
            {
                currentNodeStyle = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the CSS style class for the root bread crumb node in the trail.
        /// </summary>
        /// <remarks>
        /// When the CSS style class is <c>null</c> or empty string, the default bread crumb node CSS style class is
        /// used if available.
        /// </remarks>
        /// <value>The CSS style class for the root bread crumb node.</value>
        [Bindable(true), Category("Appearance"), DefaultValue("")]
        [Description("The CSS Style Class to use for the root node in the path.")]
        public virtual string RootNodeStyle
        {
            get
            {
                return rootNodeStyle;
            }
            set
            {
                rootNodeStyle = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the CSS style class for the bread crumb node in the trail when mouse is hovering over.
        /// </summary>
        /// <remarks>
        /// When the CSS style class is <c>null</c> or empty string, the mouse hovering effect is not used.
        /// </remarks>
        /// <value>The CSS style class for the bread crumb node when mouse is hovering over.</value>
        [Bindable(true), Category("Appearance"), DefaultValue("")]
        [Description("The CSS Style Class to use when the mouse hovers over a node in the path")]
        public virtual string MouseOverStyle
        {
            get
            {
                return mouseOverStyle;
            }
            set
            {
                mouseOverStyle = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the error disposition flag.
        /// </summary>
        /// <remarks>
        /// When exceptions occur when rendering the bread crumb trail, the exceptions are processed according to this
        /// flag.
        /// </remarks>
        /// <value>The error disposition flag.</value>
        /// <exception cref="ArgumentException">The value is not a valid error disposition.</exception>
        [Bindable(true), Category("Behavior"), DefaultValue(ErrorDisposition.SilentlyIgnore)]
        [Description("The disposition of any errors encountered during rendering.")]
        public virtual ErrorDisposition ErrorDisposition
        {
            get
            {
                return errorDisposition;
            }
            set
            {
                BreadCrumbTrailControlHelper.ValidateEnumeration(value, "value", typeof(ErrorDisposition));

                errorDisposition = value;
            }
        }

        /// <summary>
        /// Gets or sets the path direction flag.
        /// </summary>
        /// <remarks>
        /// The bread crumb trail is rendered according to this path direction flag.
        /// </remarks>
        /// <value>The path direction flag.</value>
        /// <exception cref="ArgumentException">The value is not a valid path direction.</exception>
        [Bindable(true), Category("Behavior"), DefaultValue(PathDirection.RootToCurrent)]
        [Description("The direction the path should be rendered.")]
        public virtual PathDirection PathDirection
        {
            get
            {
                return pathDirection;
            }
            set
            {
                BreadCrumbTrailControlHelper.ValidateEnumeration(value, "value", typeof(PathDirection));

                pathDirection = value;
            }
        }

        /// <summary>
        /// Gets or sets the data source of the bread crumb trail.
        /// </summary>
        /// <remarks>
        /// When the data source is available, and it is a data set, a data table or a list, its data is directly used
        /// as the bread crumb trail.
        /// </remarks>
        /// <exception cref="ArgumentException">The value is not a data set, a data table, or a list.</exception>
        /// <value>The data source of the bread crumb trail.</value>
        [Bindable(true), Category("Data"), DefaultValue(""), Description("The override for the nodes in the path")]
        public virtual object DataSource
        {
            get
            {
                return dataSource;
            }
            set
            {
                if (!((value == null) || (value is DataSet) || (value is DataTable) || (value is IList)))
                {
                    // When the data source type is not expected
                    throw new ArgumentException("Data source must be a data set, a data table or a list.", "value");
                }

                dataSource = value;
            }
        }

        /// <summary>
        /// Gets or sets the title of the current page.
        /// </summary>
        /// <remarks>
        /// When the title is <c>null</c> or empty string, the default title of the node matcher is used. The title is
        /// rendered as the content of the bread crumb node in the trail.
        /// </remarks>
        /// <value>The title of the current page.</value>
        [Bindable(true), Category("Appearance"), DefaultValue("")]
        [Description("The title override to use for the current page.")]
        public virtual string Title
        {
            get
            {
                return title;
            }
            set
            {
                title = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the key used to create the <c>INodeDiscovery</c> instance from object factory.
        /// </summary>
        /// <remarks>
        /// When the key is <c>null</c> or empty string, the default value
        /// 'TopCoder.Web.UI.WebControl.BreadCrumb.INodeDiscovery' is used.
        /// </remarks>
        /// <value>The key used to create the <c>INodeDiscovery</c> instance.</value>
        [Bindable(true), Category("Behavior"), DefaultValue("")]
        [Description("The Object Factory key to use in creating the INodeDiscovery implementation.")]
        public virtual string NodeDiscoveryKey
        {
            get
            {
                return nodeDiscoveryKey;
            }
            set
            {
                nodeDiscoveryKey = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the key used to create the <c>IPathDiscovery</c> instance from object factory.
        /// </summary>
        /// <remarks>
        /// When the key is <c>null</c> or empty string, the default value
        /// 'TopCoder.Web.UI.WebControl.BreadCrumb.IPathDiscovery' is used.
        /// </remarks>
        /// <value>The key used to create the <c>IPathDiscovery</c> instance.</value>
        [Bindable(true), Category("Behavior"), DefaultValue("")]
        [Description("The Object Factory key to use in creating the IPathDiscovery implementation.")]
        public virtual string PathDiscoveryKey
        {
            get
            {
                return pathDiscoveryKey;
            }
            set
            {
                pathDiscoveryKey = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Gets or sets the key used to create the <c>IPathWriter</c> instance from object factory.
        /// </summary>
        /// <remarks>
        /// When the key is <c>null</c> or empty string, the default value
        /// 'TopCoder.Web.UI.WebControl.BreadCrumb.IPathWriter' is used.
        /// </remarks>
        /// <value>The key used to create the <c>IPathWriter</c> instance.</value>
        [Bindable(true), Category("Behavior"), DefaultValue("")]
        [Description("The Object Factory key to use in creating the IPathWriter implementation.")]
        public virtual string PathWriterKey
        {
            get
            {
                return pathWriterKey;
            }
            set
            {
                pathWriterKey = ((value == null) || (value.Trim().Length == 0)) ? null : value;
            }
        }

        /// <summary>
        /// Represents the path separator which is rendered between two bread crumb nodes.
        /// </summary>
        private string pathSeparator = ":";

        /// <summary>
        /// Represents the CSS style class for the path separator.
        /// </summary>
        private string pathSeparatorStyle = null;

        /// <summary>
        /// Represents the CSS style class for the bread crumb node.
        /// </summary>
        private string nodeStyle = null;

        /// <summary>
        /// Represents the CSS style class for the current bread crumb node.
        /// </summary>
        private string currentNodeStyle = null;

        /// <summary>
        /// Represents the CSS style class for the root bread crumb node.
        /// </summary>
        private string rootNodeStyle = null;

        /// <summary>
        /// Represents the CSS style class for the bread crumb node which mouse hovers over.
        /// </summary>
        private string mouseOverStyle = null;

        /// <summary>
        /// Represents the error disposition flag used to decide how the exceptions are processed.
        /// </summary>
        private ErrorDisposition errorDisposition = ErrorDisposition.SilentlyIgnore;

        /// <summary>
        /// Represents the path direction flag used to decide in what direction the bread crumb trail should render.
        /// </summary>
        private PathDirection pathDirection = PathDirection.RootToCurrent;

        /// <summary>
        /// Represents the data source of the bread crumb trail used to override the current bread crumb trail.
        /// </summary>
        private object dataSource = null;

        /// <summary>
        /// Represents the title of the curren page.
        /// </summary>
        private string title = null;

        /// <summary>
        /// Represents the key in object facotry used to create <c>INodeDiscovery</c> instance.
        /// </summary>
        private string nodeDiscoveryKey = null;

        /// <summary>
        /// Represents the key in object facotry used to create <c>IPathDiscovery</c> instance.
        /// </summary>
        private string pathDiscoveryKey = null;

        /// <summary>
        /// Represents the key in object facotry used to create <c>IPathWriter</c> instance.
        /// </summary>
        private string pathWriterKey = null;

        /// <summary>
        /// Represents the current bread crumb trail extracted from the data source.
        /// </summary>
        /// <remarks>
        /// This path is directly used as the current bread crumb trail to be rendered.
        /// </remarks>
        private BreadCrumbNode[] pathOverride = null;

        /// <summary>
        /// Gets the configuration value from the configuration manager. The property name, the expected property type,
        /// and the default property value in case any error occurs.
        /// </summary>
        /// <param name="propertyName">The property name whose value is retrieved.</param>
        /// <param name="type">The expected property value type.</param>
        /// <param name="defaultValue">The default value if any error occurs.</param>
        /// <returns>The retrieved property value.</returns>
        private object GetConfigValue(string propertyName, Type type, object defaultValue)
        {
            try
            {
                ConfigManager configManager = ConfigManager.GetInstance();

                object obj = configManager.GetValue(typeof(BreadCrumbTrailControl).FullName, propertyName, type);

                // When the type is an enumeration, but the property value is not one of the defined enumeration
                // Use the default value.
                return (type.IsEnum && !Enum.IsDefined(type, obj)) ? defaultValue : obj;
            }
            catch (Exception)
            {
                return defaultValue;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>BreadCrumbTrailControl</c> class. All properties are read from the
        /// configuration if available.
        /// </summary>
        public BreadCrumbTrailControl()
        {
            PathSeparator = (string)GetConfigValue(PATHSEPARATOR_PROPERTY, typeof(string), PathSeparator);
            PathSeparatorStyle = (string)GetConfigValue(PATHSEPARATORSTYLE_PROPERTY, typeof(string),
                PathSeparatorStyle);
            NodeStyle = (string)GetConfigValue(NODESTYLE_PROPERTY, typeof(string), NodeStyle);
            CurrentNodeStyle = (string)GetConfigValue(CURRENTNODESTYLE_PROPERTY, typeof(string), CurrentNodeStyle);
            RootNodeStyle = (string)GetConfigValue(ROOTNODESTYLE_PROPERTY, typeof(string), RootNodeStyle);
            MouseOverStyle = (string)GetConfigValue(MOUSEOVERSTYLE_PROPERTY, typeof(string), MouseOverStyle);
            ErrorDisposition = (ErrorDisposition)GetConfigValue(ERRORDISPOSITION_PROPERTY, typeof(ErrorDisposition),
                ErrorDisposition);
            PathDirection = (PathDirection)GetConfigValue(PATHDIRECTION_PROPERTY, typeof(PathDirection), PathDirection);
            NodeDiscoveryKey = (string)GetConfigValue(NODEDISCOVERKEY_PROPERTY, typeof(string), NodeDiscoveryKey);
            PathDiscoveryKey = (string)GetConfigValue(PATHDISCOVERKEY_PROPERTY, typeof(string), PathDiscoveryKey);
            PathWriterKey = (string)GetConfigValue(PATHWRITERKEY_PROPERTY, typeof(string), PathWriterKey);
        }

        /// <summary>
        /// Renders the bread crumb trail for the current page.
        /// </summary>
        /// <remarks>
        /// The current bread crumb trail is obtained using the <c>INodeDiscovery</c> instance and the
        /// <c>IPathDiscovery</c> instance created from the object factory. If the current path is specified explicitly,
        /// such process is ignored and the current path is used directly. The path is then rendered using the
        /// <c>IPathWriter</c> instance created from the object factory. If any error occurs during the process, the
        /// error is processed according to the error disposition flag. If it is <c>SilentlyIgnore</c>, the exception is
        /// ignored. If it is <c>RenderException</c>, the exception is rendered to the web page. If it is
        /// <c>ThrowException</c>, the exception is thrown directly. It is also possible that the bread crumb trail
        /// cannot be found. In this case, nothing is rendered and there is no exception.
        /// </remarks>
        /// <param name="writer">The HTML text writer where the bread crumb trail is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Any error occurs when rendering the bread crumb trail, and the error
        /// disposition flag indicates throwing the exception.</exception>
        /// <exception cref="ObjectDisposedException">The HTML text writer is already closed.</exception>
        /// <exception cref="IOException">I/O error occurs when writing to the HTML text writer.</exception>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");

            BreadCrumbException[] exceptions = null;
            ObjectFactory factory = null;

            try
            {
                // Get the current path
                BreadCrumbNode[] path = pathOverride;

                if (path == null)
                {
                    // When the overridden path is not available, find the path
                    factory = ObjectFactory.GetDefaultObjectFactory(GetType().FullName);

                    // Get the current node
                    INodeDiscovery nodeDiscovery = (INodeDiscovery)factory.CreateDefinedObject(
                        (nodeDiscoveryKey == null) ? typeof(INodeDiscovery).FullName : nodeDiscoveryKey);
                    BreadCrumbNode currentNode = nodeDiscovery.GetCurrentNode(title, Page);

                    IPathDiscovery pathDiscovery = (IPathDiscovery)factory.CreateDefinedObject(
                        (pathDiscoveryKey == null) ? typeof(IPathDiscovery).FullName : pathDiscoveryKey);

                    path = pathDiscovery.GetPath(currentNode, Page);
                }

                if ((path != null) && (path.Length != 0))
                {
                    // When there is a path to be written, write the path
                    if (factory == null)
                    {
                        factory = ObjectFactory.GetDefaultObjectFactory(GetType().FullName);
                    }

                    IPathWriter pathWriter = (IPathWriter)factory.CreateDefinedObject(
                        (pathWriterKey == null) ? typeof(IPathWriter).FullName : pathWriterKey);

                    exceptions = pathWriter.WritePath(writer, path, this);

                    if ((exceptions != null) && (exceptions.Length != 0))
                    {
                        // When there is at least one error, throw the first one
                        throw exceptions[0];
                    }
                }
            }
            catch (Exception e)
            {
                BreadCrumbException exception = (e is BreadCrumbException) ? (BreadCrumbException)e :
                    new BreadCrumbException("Rendering the bread crumb path fails.", e);

                switch (ErrorDisposition)
                {
                    case ErrorDisposition.SilentlyIgnore:
                        // Ignore the exception
                        break;

                    case ErrorDisposition.ThrowException:
                        // Throw the exception
                        throw exception;

                    case ErrorDisposition.RenderException:
                        // Render each exception in the array
                        // When the exception is thrown before the WritePath, it is the only exception
                        foreach (BreadCrumbException error in
                            ((exceptions == null) ? new BreadCrumbException[] { exception } : exceptions))
                        {
                            // Render the exception in a paragraph tag.
                            // Any exception that may throw is directly thrown
                            // From the observation, HtmlTextWriter may throw IOException or ObjectDisposedException
                            // by the underlying text writer.
                            writer.RenderBeginTag(HtmlTextWriterTag.P);
                            writer.Write(HttpUtility.HtmlEncode(error.ToString()));
                            writer.RenderEndTag();
                        }

                        break;
                }
            }
        }

        /// <summary>
        /// Binds the data source to the bread crumb trail control.
        /// </summary>
        /// <remarks>
        /// The data source can be a data set, a data table or a list of <c>BreadCrumbNode</c> instances. When the data
        /// source is a data set, its first data table is used. If the data source is a data table, it must contain
        /// column "Title" and "Url", corresponding to the title and the URL in bread crumb nodes. The data table can
        /// contain no row. In this case, it is considered as if the data source is not set. When the data source is a
        /// list, it must be non-empty. It cannot contain <c>null</c>.
        /// </remarks>
        /// <exception cref="ArgumentException">The data source is a data set, but contains no data table; or the data
        /// source is a data table, but does not contain valid bread crumb nodes; or the data source is a data set, but
        /// its first data table does not contain valid bread crumb nodes; or the data source is a list, but is empty;
        /// or the data source is a list, but contains instances other than <c>BreadCrumbNode</c>; or the URL of one
        /// node in the list is <c>null</c>.</exception>
        public override void DataBind()
        {
            base.DataBind();

            BreadCrumbNode[] nodes = null;
            object source = dataSource;

            if (dataSource is DataSet)
            {
                // When the data source is a data set, use its first data table
                try
                {
                    source = ((DataSet)dataSource).Tables[0];
                }
                catch (IndexOutOfRangeException e)
                {
                    throw new ArgumentException("There is no data table in the data set.", "dataSource", e);
                }
            }

            if (source is DataTable)
            {
                // When the data source is a data table, extract its data to the nodes array
                DataTable table = (DataTable)source;

                if (table.Rows.Count != 0)
                {
                    // Only when the row count is not 0, the table is processed.
                    nodes = new BreadCrumbNode[table.Rows.Count];

                    // Extract each row
                    for (int rowIndex = 0; rowIndex < nodes.Length; ++rowIndex)
                    {
                        DataRow row = table.Rows[rowIndex];

                        try
                        {
                            nodes[rowIndex] = new BreadCrumbNode((string)row[TITLE_COLUMN], (string)row[URL_COLUMN]);
                        }
                        catch (InvalidCastException e)
                        {
                            throw new ArgumentException("The data table contains incorrect type.", "dataSource", e);
                        }
                    }
                }
            }
            else if (source is IList)
            {
                // When the data source is a list, copy it
                IList list = (IList)source;

                nodes = new BreadCrumbNode[list.Count];

                try
                {
                    // Copy the contents to the array
                    list.CopyTo(nodes, 0);
                }
                catch (InvalidCastException e)
                {
                    throw new ArgumentException("The list must contain only BreadCrumbNode.", "dataSource", e);
                }
            }

            if (nodes != null)
            {
                // When the nodes is not null, validate for null URLs
                BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(nodes, "dataSource");
            }

            pathOverride = nodes;
        }
    }
}

