/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Defines an abstract implementation of <c>INodeMatcher</c> interface to provide the functionality to hold a
    /// default title and a default URL of the node matcher. The default title and the default URL are used when a
    /// matched node title and/or URL is <c>null</c>.
    /// </summary>
    /// <remarks>
    /// Its concrete implementation must provide <c>Equals</c> and <c>GetHashCode</c> methods. This class is
    /// thread-safe, since it is immutable.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [Serializable]
    public abstract class AbstractNodeMatcher : INodeMatcher
    {
        /// <summary>
        /// Represents the default title of the matcher if a matched node does not have a title.
        /// </summary>
        private readonly string title;

        /// <summary>
        /// Represents the default URL of the matcher if a matched node does not have an URL.
        /// </summary>
        private readonly string url;

        /// <summary>
        /// Gets the default title of the node matcher.
        /// </summary>
        /// <remarks>
        /// The default title of the node matcher. It is used when the matched node title is not available.
        /// </remarks>
        /// <value>The default title of the node matcher.</value>
        public string Title
        {
            get
            {
                return title;
            }
        }

        /// <summary>
        /// Gets the default URL of the node matcher.
        /// </summary>
        /// <remarks>
        /// The default URL of the node matcher. It is used when the matched node URL is not available.
        /// </remarks>
        /// <value>The default URL of the node matcher.</value>
        public string Url
        {
            get
            {
                return url;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>AbstractNodeMatcher</c> class. The default title and the default URL are given.
        /// </summary>
        /// <param name="title">The default title of the node matcher.</param>
        /// <param name="url">The default URL of the node matcher.</param>
        /// <exception cref="ArgumentNullException"><c>title</c> or <c>url</c> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException"><c>title</c> or <c>url</c> is empty string.</exception>
        protected AbstractNodeMatcher(string title, string url)
        {
            BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(title, "title");
            BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(url, "url");

            this.title = title;
            this.url = url;
        }

        /// <summary>
        /// Gets a flag indicating whether the given node matches the criteria defined by this node matcher.
        /// </summary>
        /// <param name="node">The bread crumb node to be matched.</param>
        /// <returns><c>true</c> if <c>node</c> matches the criteria; <c>false</c> otherwise.</returns>
        /// <exception cref="ArgumentNullException">The URL of <c>node</c> is <c>null</c>.</exception>
        public abstract bool Matches(BreadCrumbNode node);
    }
}

