/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Text.RegularExpressions;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher
{
    /// <summary>
    /// Implements <c>INodeMatcher</c> interface by extending <c>AbstractNodeMatcher</c> class. It provides the
    /// functionality to match a bread crumb node according to its URL. The whole URL is matched by a regular
    /// expression.
    /// </summary>
    /// <remarks>
    /// This class is thread-safe, since it is immutable.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [Serializable]
    public class RegexNodeMatcher : AbstractNodeMatcher
    {
        /// <summary>
        /// Represents the template used to create a regular expression which can match the whole string based on
        /// another regular expression.
        /// </summary>
        private const string MATCH_WHOLE_PATTERN = "^{0}$";

        /// <summary>
        /// Represents the regular expression used to match the URL of the bread crumb node.
        /// </summary>
        private readonly string nodePattern;

        /// <summary>
        /// Represents the regular expression used to match the URL of the bread crumb node.
        /// </summary>
        private readonly Regex pattern;

        /// <summary>
        /// Gets the regular expression used to match the URL of the bread crumb node.
        /// </summary>
        /// <value>The regular expression used to match the URL of the bread crumb node.</value>
        public string NodePattern
        {
            get
            {
                return nodePattern;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>AbstractNodeMatcher</c> class. The default title, the default URL and the
        /// regular expression used to match the URL of a bread crumb node are given.
        /// </summary>
        /// <param name="title">The default title of the node matcher.</param>
        /// <param name="url">The default URL of the node matcher.</param>
        /// <param name="nodePattern">The regular expression used to match the URL of a bread crumb node.</param>
        /// <exception cref="ArgumentNullException"><c>title</c>, <c>url</c> or <c>nodePattern</c> is <c>null</c>.
        /// </exception>
        /// <exception cref="ArgumentException"><c>title</c>, <c>url</c> or <c>nodePattern</c> is empty string; or
        /// <c>nodePattern</c> is an invalid regular expression.</exception>
        public RegexNodeMatcher(string title, string url, string nodePattern)
            : base(title, url)
        {
            BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(nodePattern, "nodePattern");

            this.nodePattern = nodePattern;

            // In order to match the whole string, the first character must be '^', and the last character must be '$'.
            // First, check if the pattern itself is legal. It must be done, since the last character may be '\',
            // which escapes the '$' character and becomes a legal regular expression.
            // If '^' and '$' are not added, non-greedy matching may fail, though it should match the whole URL.
            new Regex(nodePattern);

            this.pattern = new Regex(string.Format(MATCH_WHOLE_PATTERN, nodePattern), RegexOptions.Singleline);
        }

        /// <summary>
        /// Matches the URL of the given bread crumb node using the regular expression.
        /// </summary>
        /// <remarks>
        /// When the whole URL matches the regular expression, the bread crumb node is considered as matched.
        /// </remarks>
        /// <param name="node">The bread crumb node to be matched.</param>
        /// <returns><c>true</c> if the URL of <c>node</c> matches the regular expression; <c>false</c> otherwise.
        /// </returns>
        /// <exception cref="ArgumentNullException">The URL of <c>node</c> is <c>null</c>.</exception>
        public override bool Matches(BreadCrumbNode node)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(node.Url, "node.Url");

            return pattern.IsMatch(node.Url);
        }

        /// <summary>
        /// Gets a flag indicating whether this instance is the same as the given instance.
        /// </summary>
        /// <remarks>
        /// When the given instance is exactly a <c>RegexNodeMatcher</c> instance, and its default title, default URL,
        /// and regular expression are the same, they are considered as the same.
        /// </remarks>
        /// <param name="obj">The instance to be compared.</param>
        /// <returns><c>true</c> if <c>obj</c> is the same as this instance; <c>false</c> otherwise.</returns>
        public override bool Equals(object obj)
        {
            if ((obj != null) && (obj.GetType().Equals(GetType())))
            {
                // When the object is a regular expression node matcher
                // Note that obj must be exactly a RegexNodeMatcher instance, not a sub-type.
                RegexNodeMatcher matcher = (RegexNodeMatcher)obj;

                return (matcher.Url == Url) && (matcher.Title == Title) && (matcher.nodePattern == nodePattern);
            }

            // Otherwise, it either is null or an instance of another type
            return false;
        }

        /// <summary>
        /// Gets the hash code of this instance.
        /// </summary>
        /// <remarks>
        /// This hash code is computed using the regular expression, the default title and the default URL.
        /// </remarks>
        /// <returns>The hash code of this instance.</returns>
        public override int GetHashCode()
        {
            return nodePattern.GetHashCode() ^ Title.GetHashCode() ^ Url.GetHashCode();
        }
    }
}

