/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Defines a pair of bread crumb node and the bread crumb node matcher which matches the node. It is used to build
    /// the current bread crumb trail according to the previous bread crumb trail.
    /// </summary>
    /// <remarks>
    /// This class is thread-safe, since it is immutable.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [Serializable]
    internal class NodePair
    {
        /// <summary>
        /// Represents the bread crumb node in the pair.
        /// </summary>
        private readonly BreadCrumbNode node;

        /// <summary>
        /// Represents the bread crumb node matcher in the pair, which matches the node.
        /// </summary>
        private readonly INodeMatcher nodeMatcher;

        /// <summary>
        /// Gets the bread crumb node in the pair.
        /// </summary>
        /// <remarks>
        /// When the title of the bread crumb node is <c>null</c>, the default title in the node matcher is used.
        /// </remarks>
        /// <value>The bread crumb node in the pair.</value>
        public BreadCrumbNode Node
        {
            get
            {
                return node;
            }
        }

        /// <summary>
        /// Gets the bread crumb node matcher in the pair, which matches the node.
        /// </summary>
        /// <value>The bread crumb node matcher in the pair.</value>
        public INodeMatcher NodeMatcher
        {
            get
            {
                return nodeMatcher;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>NodePair</c> class. The bread crumb node and the node matcher in the pair are
        /// given. When the title of the bread crumb node is <c>null</c> or empty string, the default title in the node
        /// matcher is used instead.
        /// </summary>
        /// <param name="node">The bread crumb node in the pair.</param>
        /// <param name="nodeMatcher">The bread crumb node matcher in the pair.</param>
        /// <exception cref="ArgumentNullException"><c>nodeMatcher</c> is <c>null</c>; or the URL of <c>node</c> is
        /// <c>null</c>.</exception>
        internal NodePair(BreadCrumbNode node, INodeMatcher nodeMatcher)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(nodeMatcher, "nodeMatcher");
            BreadCrumbTrailControlHelper.ValidateNotNull(node.Url, "node.Url");

            this.node = (node.Title == null) ? new BreadCrumbNode(nodeMatcher.Title, node.Url) : node;
            this.nodeMatcher = nodeMatcher;
        }
    }
}

