/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// Implements <c>IPathWriter</c> interface by extending the <c>AbstractPathWriter</c> class. This class provides
    /// the functionality to render the bread crumb node as an HTML anchor. The content of the anchor is the title of
    /// the bread crumb node. The hyperlink of the anchor is the URL of the bread crumb node.
    /// </summary>
    /// <remarks>
    /// This class is thread-safe, since it is stateless.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class AnchorPathWriter : AbstractPathWriter
    {
        /// <summary>
        /// Represents the attribute of the mouse over event in HTML.
        /// </summary>
        private const string ONMOUSEOVER_ATTRIBUTE = "onmouseover";

        /// <summary>
        /// Represents the attribute of the mouse out event in HTML.
        /// </summary>
        private const string ONMOUSEOUT_ATTRIBUTE = "onmouseout";

        /// <summary>
        /// Represents the javascript used to change the class of an HTML element.
        /// </summary>
        private const string CLASS_JAVASCRIPT = "this.className='{0}'";

        /// <summary>
        /// Represents the default title when the title of a bread crumb node is <c>null</c>.
        /// </summary>
        private const string UNKNOWN_TITLE = "Unknown";

        /// <summary>
        /// Creates a new instance of <c>AnchorPathWriter</c> class.
        /// </summary>
        public AnchorPathWriter()
        {
        }

        /// <summary>
        /// Renders a single bread crumb node in the HTML web page. The style of the node and the style when mouse
        /// hovers over the content are given.
        /// </summary>
        /// <remarks>
        /// When the node style or the mouse over style is <c>null</c>, the node style and/or the mouse over style is
        /// not used. The content of the HTML anchor is the title of the bread crumb node. The hyperlink of the HTML
        /// anchor is the URL of the bread crumb node.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the bread crumb node is rendered.</param>
        /// <param name="node">The bread crumb node to be rendered.</param>
        /// <param name="nodeStyle">The node CSS style class used to render the node.</param>
        /// <param name="mouseOverStyle">The CSS style class when the mouse hovers over the rendered node.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>; or the URL of <c>node</c> is
        /// <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the bread crumb node fails.</exception>
        protected override void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");
            BreadCrumbTrailControlHelper.ValidateNotNull(node.Url, "node.Url");

            // Treat empty string as null
            nodeStyle = ((nodeStyle == null) || (nodeStyle.Trim().Length == 0)) ? null : nodeStyle;
            mouseOverStyle = ((mouseOverStyle == null) || (mouseOverStyle.Trim().Length == 0)) ? null : mouseOverStyle;

            if (nodeStyle != null)
            {
                // When the node style exists, add the class tag
                writer.AddAttribute(HtmlTextWriterAttribute.Class, nodeStyle);
            }

            if (mouseOverStyle != null)
            {
                // When the mouse over style exists, add the event
                writer.AddAttribute(ONMOUSEOVER_ATTRIBUTE, string.Format(CLASS_JAVASCRIPT, mouseOverStyle));

                // When the node style does not exist, use empty class.
                writer.AddAttribute(ONMOUSEOUT_ATTRIBUTE,
                    string.Format(CLASS_JAVASCRIPT, (nodeStyle != null) ? nodeStyle : ""));
            }

            // Add the URL of the anchor
            writer.AddAttribute(HtmlTextWriterAttribute.Href, node.Url);

            try
            {
                // Render the anchor
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                writer.Write(HttpUtility.HtmlEncode((node.Title == null) ? UNKNOWN_TITLE : node.Title));
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                // AddAttribute would not throw any exception. Only write or render may throw.
                throw new BreadCrumbException("Rendering the bread crumb node fails.", e);
            }
        }
    }
}

