/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher
{
    /// <summary>
    /// Tests the functionality and error cases of <c>RegexNodeMatcher</c> class. Since this class is immutable, the
    /// property getters are tested together with the constructor. It only tests members defined in this class, since
    /// the members defined in its super class have already been tested.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class RegexNodeMatcherTestCase
    {
        /// <summary>
        /// Represents the <c>RegexNodeMatcher</c> instance used in tests.
        /// </summary>
        private INodeMatcher matcher;

        /// <summary>
        /// Sets up the test environment. The test instance is created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            matcher = new RegexNodeMatcher("title", "url", @"abc(.*)efg(\1)");
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            matcher = null;
        }

        /// <summary>
        /// Tests <c>RegexNodeMatcher(string, string, string)</c> when the regular expression is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestRegexNodeMatcherStringStringStringPatternNull()
        {
            new RegexNodeMatcher("title", "url", null);
        }

        /// <summary>
        /// Tests <c>RegexNodeMatcher(string, string, string)</c> when the regular expression is empty string. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestRegexNodeMatcherStringStringStringPatternEmpty()
        {
            new RegexNodeMatcher("title", "url", "  ");
        }

        /// <summary>
        /// Tests <c>RegexNodeMatcher(string, string, string)</c> when the regular expression is invalid. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestRegexNodeMatcherStringStringStringPatternInvalid()
        {
            new RegexNodeMatcher("title", "url", @"\");
        }

        /// <summary>
        /// Tests accuracy of <c>RegexNodeMatcher(string, string, string)</c>. The title, URL and the regular expression
        /// should be set as given. These are verified by the property getters. This is also a test case for the
        /// property getters.
        /// </summary>
        [Test]
        public void TestRegexNodeMatcherStringStringStringAccuracy()
        {
            RegexNodeMatcher matcher = new RegexNodeMatcher("title", "url", ".*");

            // Verify
            Assert.AreEqual("title", matcher.Title, "The title should be set as 'title'.");
            Assert.AreEqual("url", matcher.Url, "The URL should be set as 'url'.");
            Assert.AreEqual(".*", matcher.NodePattern, "The regular expression should be set as '.*'.");
        }

        /// <summary>
        /// Tests <c>Matches(BreadCrumbNode)</c> when the URL of the node is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchesBreadCrumbNodeUrlNull()
        {
            matcher.Matches(new BreadCrumbNode());
        }

        /// <summary>
        /// Tests accuracy of <c>Matches(BreadCrumbNode)</c> when the URL of the bread crumb node does not match the
        /// regular expression. It should return <c>false</c>.
        /// </summary>
        [Test]
        public void TestMatchesBreadCrumbNodeNotMatchAccuracy()
        {
            Assert.IsFalse(matcher.Matches(new BreadCrumbNode("title", "anything abcsomthingefgsomthing")),
                "The URL does not match the regular expression.");
        }

        /// <summary>
        /// Tests accuracy of <c>Matches(BreadCrumbNode)</c> when the URL of the bread crumb node matches the regular
        /// expression. It should return <c>true</c>.
        /// </summary>
        [Test]
        public void TestMatchesBreadCrumbNodeMatchAccuracy()
        {
            Assert.IsTrue(matcher.Matches(new BreadCrumbNode("title", "abcsomthingelseefgsomthingelse")),
                "The URL matches the regular expression.");
        }

        /// <summary>
        /// Tests accuracy of <c>Equals(object)</c> when the given instance is <c>null</c>. It should return
        /// <c>false</c>.
        /// </summary>
        [Test]
        public void TestEqualsObjectNullAccuracy()
        {
            Assert.IsFalse(matcher.Equals(false), "The matcher should not be the same as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Equals(object)</c> when the given instance is another type. It should return
        /// <c>false</c>.
        /// </summary>
        [Test]
        public void TestEqualsObjectOtherTypeAccuracy()
        {
            Assert.IsFalse(matcher.Equals(new object()),
                "The matcher should not be the same as an instance of another type.");
        }

        /// <summary>
        /// Tests accuracy of <c>Equals(object)</c> when the given instance is a regular expression node matcher.
        /// However, the title is not the same. It should return <c>false</c>.
        /// </summary>
        [Test]
        public void TestEqualsObjectMatcherTitleNotSameAccuracy()
        {
            Assert.IsFalse(matcher.Equals(new RegexNodeMatcher("title1", "url", @"abc(.*)efg(\1)")),
                "The matcher should not be the same as a regular expression matcher with different title.");
        }

        /// <summary>
        /// Tests accuracy of <c>Equals(object)</c> when the given instance is a regular expression node matcher.
        /// However, the URL is not the same. It should return <c>false</c>.
        /// </summary>
        [Test]
        public void TestEqualsObjectMatcherURLNotSameAccuracy()
        {
            Assert.IsFalse(matcher.Equals(new RegexNodeMatcher("title", "url1", @"abc(.*)efg(\1)")),
                "The matcher should not be the same as a regular expression matcher with different URL.");
        }

        /// <summary>
        /// Tests accuracy of <c>Equals(object)</c> when the given instance is a regular expression node matcher.
        /// However, the regular expression is not the same. It should return <c>false</c>.
        /// </summary>
        [Test]
        public void TestEqualsObjectMatcherPatternNotSameAccuracy()
        {
            Assert.IsFalse(matcher.Equals(new RegexNodeMatcher("title", "url", @"abc(.*)efg(abc)")),
                "The matcher should not be the same as a regular expression matcher with different pattern.");
        }

        /// <summary>
        /// Tests accuracy of <c>Equals(object)</c> when the given instance is a regular expression node matcher.
        /// The title, the URL and the regular expression are the same. It should return <c>true</c>.
        /// </summary>
        [Test]
        public void TestEqualsObjectMatcherSameAccuracy()
        {
            Assert.IsTrue(matcher.Equals(new RegexNodeMatcher("title", "url", @"abc(.*)efg(\1)")),
                "The matchers should be the same.");
        }

        /// <summary>
        /// Tests accuracy of <c>GetHashCode()</c>. Two same instances should return the same hash code. However, two
        /// different instances are not guaranteed to return different hash codes.
        /// </summary>
        [Test]
        public void TestGetHashCodeAccuracy()
        {
            INodeMatcher newMatcher = new RegexNodeMatcher("title", "url", @"abc(.*)efg(\1)");

            // Verify the hash code
            Assert.AreEqual(matcher.GetHashCode(), newMatcher.GetHashCode(),
                "The same regular expression matchers should return the same hash code.");
        }
    }
}

