/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.Web.UI;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// Tests the functionality and error cases of <c>OrderedListPathWriter</c> class. The protected methods are tested
    /// via reflection.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class OrderedListPathWriterTestCase
    {
        /// <summary>
        /// Represents the <c>OrderedListPathWriter</c> instance used in tests.
        /// </summary>
        private IPathWriter writer;

        /// <summary>
        /// Represents the <c>HtmlTextWriter</c> instance used in tests.
        /// </summary>
        private HtmlTextWriter htmlWriter;

        /// <summary>
        /// Represents the <c>StringWriter</c> from which the <c>HtmlTextWriter</c> is created.
        /// </summary>
        private TextWriter stringWriter;

        /// <summary>
        /// Sets up the test environment. The test instances are created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            writer = new OrderedListPathWriter();
            stringWriter = new StringWriter();
            htmlWriter = new HtmlTextWriter(stringWriter);
        }

        /// <summary>
        /// Cleans up the test environment. The test instances are disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            writer = null;
            htmlWriter.Close();
            htmlWriter = null;
            stringWriter = null;
        }

        /// <summary>
        /// Tests accuracy of <c>OrderedListPathWriter()</c>. It is a very simple constructor. Only verify there is a
        /// new instance is created.
        /// </summary>
        [Test]
        public void TestOrderedListPathWriterAccuracy()
        {
            Assert.IsNotNull(new OrderedListPathWriter(), "A new instance should be created.");
        }

        /// <summary>
        /// Tests <c>BeginPath(HtmlTextWriter)</c> when the given writer is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestBeginPathHtmlTextWriterNull()
        {
            TestHelper.Invoke(writer, "BeginPath", new object[] { null });
        }

        /// <summary>
        /// Tests <c>BeginPath(HtmlTextWriter)</c> when rendering to the HTML writer fails. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestBeginPathHtmlTextWriterRenderError()
        {
            htmlWriter.Close();

            TestHelper.Invoke(writer, "BeginPath", htmlWriter);
        }

        /// <summary>
        /// Tests accuracy of <c>BeginPath(HtmlTextWriter)</c>. The OL HTML tag should be written. This is verified by
        /// comparing the render result to a reference result.
        /// </summary>
        [Test]
        public void TestBeginPathHtmlTextWriterAccuracy()
        {
            TestHelper.Invoke(writer, "BeginPath", htmlWriter);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<ol>\r\n", stringWriter.ToString(), "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests <c>EndPath(HtmlTextWriter)</c> when the given writer is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestEndPathHtmlTextWriterNull()
        {
            TestHelper.Invoke(writer, "EndPath", new object[] { null });
        }

        /// <summary>
        /// Tests <c>EndPath(HtmlTextWriter)</c> when rendering to the HTML writer fails. Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestEndPathHtmlTextWriterRenderError()
        {
            TestHelper.Invoke(writer, "BeginPath", htmlWriter);

            htmlWriter.Close();

            TestHelper.Invoke(writer, "EndPath", htmlWriter);
        }

        /// <summary>
        /// Tests accuracy of <c>EndPath(HtmlTextWriter)</c>. The OL HTML tag should be written. This is verified by
        /// comparing the render result to a reference result.
        /// </summary>
        [Test]
        public void TestEndPathHtmlTextWriterAccuracy()
        {
            TestHelper.Invoke(writer, "BeginPath", htmlWriter);
            TestHelper.Invoke(writer, "EndPath", htmlWriter);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<ol>\r\n\r\n</ol>", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the HTML text writer is
        /// <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringWriterNull()
        {
            TestHelper.Invoke(writer, "WriteNode", null, new BreadCrumbNode("title", "url"), null, null);
        }

        /// <summary>
        /// Tests <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the rendering the node fails.
        /// Expect BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringRenderError()
        {
            htmlWriter.Close();

            TestHelper.Invoke(writer, "WriteNode", htmlWriter, new BreadCrumbNode("title", "url"), null, null);
        }

        /// <summary>
        /// Tests <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the URL of the node is
        /// <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringNodeURLNull()
        {
            TestHelper.Invoke(writer, "WriteNode", htmlWriter, new BreadCrumbNode(), null, null);
        }

        /// <summary>
        /// Tests accuracy of <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the node style and
        /// the mouse over style are <c>null</c>. The rendered node should not use any style. It should be rendered as
        /// an HTML anchor wrapped by a list item tag. This is verified by comparing the rendered result to a reference
        /// result.
        /// </summary>
        [Test]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringNoStyleAccuracy()
        {
            TestHelper.Invoke(writer, "WriteNode", htmlWriter, new BreadCrumbNode("title", "url"), null, null);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<li><a href=\"url\">title</a></li>", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the node style is
        /// given, but the mouse over style is <c>null</c>. The rendered node should use the node style. It should be
        /// rendered as an HTML anchor wrapped by a list item tag. This is verified by comparing the rendered result to
        /// a reference result.
        /// </summary>
        [Test]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringNodeStyleAccuracy()
        {
            TestHelper.Invoke(writer, "WriteNode", htmlWriter, new BreadCrumbNode("title", "url"), "node", null);

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<li><a class=\"node\" href=\"url\">title</a></li>", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the mouse over style
        /// is given, but the node style is <c>null</c>. The rendered node should use the mouse over style. It should be
        /// rendered as an HTML anchor wrapped by a list item tag. This is verified by comparing the rendered result to
        /// a reference result.
        /// </summary>
        [Test]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringMouseStyleAccuracy()
        {
            TestHelper.Invoke(writer, "WriteNode", htmlWriter, new BreadCrumbNode("title", "url"), null, "mouse");

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<li><a onmouseover=\"this.className='mouse'\" onmouseout=\"this.className=''\" " +
                "href=\"url\">title</a></li>", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WriteNode(HtmlTextWriter, BreadCrumbNode, string, string)</c> when the mouse over style
        /// and the node style are given. The rendered node should use the both styles. It should be rendered as an HTML
        /// anchor wrapped by a list item tag. This is verified by comparing the rendered result to a reference result.
        /// </summary>
        [Test]
        public void TestWriteNodeHtmlTextWriterBreadCrumbNodeStringStringAllStyleAccuracy()
        {
            TestHelper.Invoke(writer, "WriteNode", htmlWriter, new BreadCrumbNode("title", "url"), "node", "mouse");

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("<li><a class=\"node\" onmouseover=\"this.className='mouse'\" " +
                "onmouseout=\"this.className='node'\" href=\"url\">title</a></li>", stringWriter.ToString(),
                "The rendered result should be the same as expected.");
        }

        /// <summary>
        /// Tests accuracy of <c>WriteSeparator(HtmlTextWriter, BreadCrumbTrailControl)</c>. It should render nothing
        /// to the HTML writer, even if the path separator in the control is available.
        /// </summary>
        [Test]
        public void TestWriteSeparatorHtmlTextWriterBreadCrumbTrailControlAccuracy()
        {
            TestHelper.Invoke(writer, "WriteSeparator", htmlWriter, new BreadCrumbTrailControl());

            htmlWriter.Flush();

            // Verify
            Assert.AreEqual("", stringWriter.ToString(), "Writing separator should write nothing.");
        }
    }
}

